﻿Type=StaticCode
Version=2.71
@EndOfDesignText@

'Code module
'
'HotSpotLib - Steve Laming
'Version 1.1 - Added HotCircle
'Version 1.2 - Added HotSpotFindLast
'Version 1.3 - Added Method IsInitialized
'Version 1.4 - Added Methods GetRectangle and ClearAllHotSpots
'            - Removed DIP conversion within the module, it should be specified in method calls
'Version 1.5 - Fixed bug where HotSpots were added twice when app was left then returned to.
'
'
'Subs in this code module will be accessible from all modules.
Private Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	
	Private HotSpotsList As List
	Type HotSpotType(ID As Int,HSXIndex As Int,HSYIndex As Int,Rectangle As Rect,Valid As Boolean,Tag As Object,Type As String,xCoords() As Int,yCoords() As Int)
	Dim Initialized As Boolean = False
End Sub
Sub Initialize
	HotSpotsList.Initialize
	Initialized = True
End Sub
'Create a HotPoly
Sub AddHotPoly(xCoords() As Int,yCoords() As Int,cTag As String) As Int
	
	'Find the Minimum and Maximum of xCoords
	Dim cLeft,cRight,cTop,cBottom As Int 
	
	Dim t As Int
	cLeft = xCoords(0)
	For i = 1 To xCoords.Length-1
		t=xCoords(i)
		If t < cLeft Then cLeft = t
		If t > cRight Then cRight = t
	Next
	
	cTop = yCoords(0)
	For i = 1 To yCoords.Length-1
		t=yCoords(i)
		If t < cTop Then cTop = t
		If t > cBottom Then cBottom = t
	Next
	
	Dim Id As Int = AddHotSpot(cLeft,cTop,cRight,cBottom,cTag)
	Dim Pos As Int =FindIndex(Id)
	Dim HS As HotSpotType = HotSpotsList.Get(Pos)
	
	'Edit HotSpot to create a poly
	HS.xCoords=xCoords
	HS.yCoords=yCoords
	HS.Type="P"
	
	Return HS.Id
	
End Sub
'Create a hotspot
'HotSpotLib.AddHotSpot(100,150,130,180,"HS 1") will create a hotspot
'at X=100, Y=150 with a width of 30 and a height of 30
Sub AddHotSpot(cLeft As Int,cTop As Int, cRight As Int,cBottom As Int,cTag As Object) As Int

	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return
	End If
	
	Dim R As Rect
	R.Initialize(cLeft,cTop,cRight,cBottom)
	
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=cLeft
	H.HSYIndex=cTop
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type ="S"
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'Create a hotspot from a rect
'HotSpotLib.AddHotSpot(Rect,"HS 1") will create a hotspot
'with the dimensions of the rectangle passed
Sub AddHotSpotRect(R As Rect,cTag As Object) As Int

	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return
	End If
	
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=R.Left
	H.HSYIndex=R.Top
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type ="S"
	Log(H)
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'HotSpotLib.AddHotCircle(20Dip,20Dip,80Dip,"Tag4") will create a hotspot at
'Left 20dip,Top 20Dip, Diameter 80dip. Center = 20+(80/2) = 60.
Sub AddHotCircle(cLeft As Int,cTop As Int, cDiameter As Int,cTag As Object) As Int
	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return
	End If
	Dim R As Rect
	R.Initialize(cLeft,cTop,cLeft+cDiameter,cTop+cDiameter)
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=cLeft
	H.HSYIndex=cTop
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type="C"
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'Turns off hotspot, returns the ID if successful or -1 if it doesnt exist
Sub DeactivateHotSpot(ID As Int) As Int
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Valid=False
		Return ID
	Else
		Return Pos
	End If
End Sub
'Edit an existing HotSpot
Sub EditHotSpot(mID As Int,mLeft As Int,mTop As Int, mRight As Int,mBottom As Int,mTag As Object) As Int
	
	Dim Pos As Int
	Pos=FindIndex(mID)
	If Pos <> -1 Then
		Dim R As Rect
		R.Initialize(mLeft,mTop,mRight,mBottom)
		Dim H As HotSpotType
		H=HotSpotsList.get(Pos)
		
		If H.Type = "P" Then
			Log("Cannot edit a Polygon")
			Return -1
		End If
		
		H.Initialize
		H.HSXIndex=mLeft
		H.HSYIndex=mTop
		H.Rectangle=R
		H.Valid=True
		H.Tag=mTag
		H.ID=mID
		HotSpotsList.RemoveAt(Pos)
		HotSpotsList.Add(H)
		Return H.ID
	End If
End Sub
'Turns on hotspot, returns the ID if successful or -1 if it doesnt exist
Sub ReActivateHotspot(ID As Int) As Int
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Valid=True
		Return ID
	Else
		Return Pos
	End If
End Sub
'Change HotSpotType 'S'quare or 'C'ircle
Sub ChangeType(ID As Int,mType As String) As Int
	If mType = "P" Then 
		Log("Cannot change type to Poly")
		Return -1
	End If
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Type=mType
		Return ID
	Else
		Return Pos
	End If
End Sub
'Readable output of object from ID
Sub ToStringID(ID As Int) As String
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		Return ("ID "&HS.ID&" X "&HS.HSXIndex&" Y "&HS.HSYIndex&" Rect "&HS.Rectangle&" Valid "&HS.Valid&" Tag "&HS.Tag)
	Else
		Return Pos
	End If
End Sub
'Readable output of object from Object
Sub ToStringHS(HS As HotSpotType) As String
	Return ("ID "&HS.ID&" X "&HS.HSXIndex&" Y "&HS.HSYIndex&" Rect "&HS.Rectangle&" Valid "&HS.Valid&" Tag "&HS.Tag)
End Sub
'Find the HotSpot from the ID via Binary search

Private Sub FindIndex(Val As Int) As Int
	Dim iMax,iMin,iMid As Int
	HotSpotsList.SortType("ID",True)
	iMax=HotSpotsList.Size-1
	iMin=0
	
	'Set up for first iteration
	iMid=iMin+((iMax-iMin)/2)
	Dim HS As HotSpotType = HotSpotsList.get(iMid)
	
	'Start
	Do While iMin < iMax
		'1st or second half
		If HS.ID < Val Then
			iMin=iMid+1
		Else
			iMax=iMid
		End If
		'Set pointer to half way for subsequent iterations
		iMid=iMin+((iMax-iMin)/2)
		Dim HS As HotSpotType = HotSpotsList.get(iMid)
	Loop
	
	'Check if found
	If iMax=iMin AND HS.ID=Val Then
		Return iMin
	Else
		Return -1
		End If
End Sub
'Finds a hot spot by searching the x axis then the y axis
Sub FindHotSpot(X As Float, Y As Float) As HotSpotType

	'Sort in required order Y within X
	HotSpotsList.SortType("HSYIndex",True)
	HotSpotsList.SortType("HSXIndex",True)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	Dim I As Int = 0
	Do While I < HotSpotsList.Size
		'Get the first hotspot to check
		Dim HS As HotSpotType = HotSpotsList.get(I)

		'Have we deactivated this one?
		If HS.Valid = False Then
			I=I+1
			Continue
		End If

		'Not Found if we have passed the required position
		If HS.Rectangle.Left > X Then Return NotFound
		
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X Then
			'X is correct
			Dim j As Int = 0
		
			'Check Y Axis - Make sure we  are still within the X Axis and haven't run out of hotspots
			Do While  I+j < HotSpotsList.Size AND HS.Rectangle.Left <= X
				
				'Fit's the Y Axis
				If HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then
										
					Select HS.Type
					
						Case "S"
							Return HS
						
						Case "C"
							If InCircle(HS.Rectangle,X,Y) Then Return HS
							Return NotFound
						Case "P"
							If pnpoly(HS.xCoords.length, HS.xCoords, HS.yCoords, X, Y) Then
                                    Return HS
                            End If
							Return NotFound
							
					End Select
				End If
				j=j+1
				'Read the next hotspot
				If I+j < HotSpotsList.Size Then HS = HotSpotsList.get(I+j)
			Loop
			
			'Finished checking subsequent Y Axis and should now have finished the search but send it back anyway
			I=I+j
		End If
		I=I+1
	Loop
	
	'Found nothing
	Return NotFound
End Sub
'Finds all hot spots by looking at each one, returns a list of found hotspots or a list with an uninitialised hotspot
Sub FindHotSpotAll(X As Float, Y As Float) As List

	Dim HSList As List
	HSList.Initialize
	Dim NotFoundList As List
	NotFoundList.Initialize

	'Sort not required as will check all
		
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then
			Select HS.Type
						
				Case "S"
					HSList.Add(HS)
				
				Case "C"
					If InCircle(HS.Rectangle,X,Y) Then HSList.Add(HS)
					
					
				Case "P"
					If pnpoly(HS.xCoords.length, HS.xCoords, HS.yCoords, X, Y) Then HSList.Add(HS)
	                
			End Select
		End If
	Next
		
	If HSList.Size > 0 Then Return HSList
	'Found nothing
	Return NotFoundList
End Sub
'Find a hotspot by looking through a list in descending order of creation, will return the last hotspot created that is touched.
Sub FindHotSpotLast(X As Float, Y As Float) As HotSpotType

	'Sort in required reverse creation order
	HotSpotsList.SortType("ID",False)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Valid AND HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then
			Select HS.Type
				Case "S"
					Return HS
				
				Case "C"
					If InCircle(HS.Rectangle,X,Y) Then Return HS
					Return NotFound
				Case "P"
					If pnpoly(HS.xCoords.length, HS.xCoords, HS.yCoords, X, Y) Then
                            Return HS
                    End If
					Return NotFound
					
			End Select
		End If
	Next
	Return NotFound
End Sub
'Find a hotspot by looking through a list in order of creation, will return the first hotspot created that is touched.
Sub FindHotSpotFirst(X As Float, Y As Float) As HotSpotType

	'Sort in required reverse creation order
	HotSpotsList.SortType("ID",True)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Valid AND HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then 
		
			Select HS.Type
						
				Case "S"
					Return HS
				
				Case "C"
					If InCircle(HS.Rectangle,X,Y) Then Return HS
					Return NotFound
				Case "P"
					If pnpoly(HS.xCoords.length, HS.xCoords, HS.yCoords, X, Y) Then
	                        Return HS
	                End If
					Return NotFound
					
			End Select
		End If
	Next
	Return NotFound
End Sub

Private Sub InCircle(R As Rect,X As Float,Y As Float) As Boolean

	Dim Distance As Float
	Distance = Sqrt(Power(X-R.CenterX,2)+Power(Y-R.CenterY,2))
	If Distance > (R.Right-R.Left)/2 Then Return False
	Return True
End Sub
''Returns the current initialized state of the lib.  To initialize, add a hotspot.
'Sub IsInitialized As Boolean
'	Return HotSpotsList.IsInitialized
'End Sub
''Returns a rect o the hotspot
Sub GetRectangle(ID As Int) As Rect
	Dim HS As HotSpotType = HotSpotsList.Get(FindIndex(ID))
	Return HS.Rectangle
End Sub
'Clear all hotspots from the list
Sub ClearHotSpots
	HotSpotsList.Clear
End Sub
'Returns the current initialized state of the lib.
Sub IsInitialized As Boolean
	Return Initialized
End Sub
Private Sub pnpoly(nvert As Int, vertx() As Int, verty() As Int, testx As Int, testy As Int) As Boolean

    Dim i, j As Int
	Dim c As Boolean = False
    'For (i = 0, j = nvert - 1; i < nvert; j = i++) {
	j=nvert-1
	For i = 0 To nvert -1
        If (((verty(i) > testy) <> (verty(j) > testy)) AND (testx < (vertx(j) - vertx(i)) * (testy - verty(i)) / (verty(j) - verty(i)) + vertx(i))) Then
            c = Not(c)
        End If
		j=i
    Next
	
    Return c
End Sub
Sub DrawHotSpot (canv As Canvas, Id As Int, Colour As Int)
	Dim Pos As Int = FindIndex(Id)
	Dim HS As HotSpotType = HotSpotsList.Get(Pos)
	
	If HS.Type <> "S" Then 
		LogColor("Error in DrawHotSpot, Hot Spot is not a Standard hot spot",Colors.Cyan)
	End If
	
	canv.DrawRect(HS.Rectangle,Colour,True,0)
	
End Sub
Sub DrawHotCircle(canv As Canvas, Id As Int, Colour As Int)
	Dim Pos As Int = FindIndex(Id)
	Dim HS As HotSpotType = HotSpotsList.Get(Pos)
	
	If HS.Type <> "C" Then 
		LogColor("Error in DrawHotCircle, Hot Spot is not a circle",Colors.Cyan)
	End If
	
	canv.DrawCircle(HS.Rectangle.CenterX,HS.Rectangle.CenterY,HS.Rectangle.CenterX-HS.Rectangle.Left,Colour,True,0)
	
End Sub
Sub DrawHotPoly (canv As Canvas, Id As Int, Colour As Int)
	Dim Pos As Int = FindIndex(Id)
	Dim HS As HotSpotType = HotSpotsList.Get(Pos)
	
	If HS.Type <> "P" Then 
		LogColor("Error in DrawHotPoly, Hot Spot is not a Polygon",Colors.Cyan)
	End If
	
	Dim P As Path
	P.Initialize(HS.xCoords(0),HS.yCoords(0))
	For i = 1 To HS.xCoords.Length-1
		P.LineTo(HS.xCoords(i),HS.yCoords(i))
	Next
	canv.DrawPath(P,Colour,True,0)
	
End Sub